# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_mode import ApiMode
from stripe._stripe_object import StripeObject
from stripe._stripe_response import StripeResponse
from stripe.v2._amount import Amount
from stripe.v2.core._event import Event, EventNotification
from typing import Any, Dict, Optional, cast
from typing_extensions import Literal, TYPE_CHECKING, override

if TYPE_CHECKING:
    from stripe._api_requestor import _APIRequestor


class V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventNotification(
    EventNotification,
):
    LOOKUP_TYPE = (
        "v2.core.health.issuing_authorization_request_timeout.resolved"
    )
    type: Literal[
        "v2.core.health.issuing_authorization_request_timeout.resolved"
    ]

    @override
    def fetch_event(
        self,
    ) -> "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent":
        return cast(
            "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent",
            super().fetch_event(),
        )

    @override
    async def fetch_event_async(
        self,
    ) -> "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent":
        return cast(
            "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent",
            await super().fetch_event_async(),
        )


class V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent(Event):
    LOOKUP_TYPE = (
        "v2.core.health.issuing_authorization_request_timeout.resolved"
    )
    type: Literal[
        "v2.core.health.issuing_authorization_request_timeout.resolved"
    ]

    class V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventData(
        StripeObject,
    ):
        class Impact(StripeObject):
            approved_amount: Optional[Amount]
            """
            Estimated aggregated amount for the approved requests.
            """
            approved_impacted_requests: Optional[int]
            """
            The number of approved requests which are impacted.
            """
            declined_amount: Optional[Amount]
            """
            Estimated aggregated amount for the declined requests.
            """
            declined_impacted_requests: Optional[int]
            """
            The number of declined requests which are impacted.
            """

        grouping_key: str
        """
        The grouping key for the alert.
        """
        impact: Impact
        """
        The user impact.
        """
        resolved_at: str
        """
        The time when the user experience has returned to expected levels.
        """
        started_at: str
        """
        The time when impact on the user experience was first detected.
        """
        summary: str
        """
        A short description of the alert.
        """
        _inner_class_types = {"impact": Impact}

    data: V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventData
    """
    Data for the v2.core.health.issuing_authorization_request_timeout.resolved event
    """

    @classmethod
    def _construct_from(
        cls,
        *,
        values: Dict[str, Any],
        last_response: Optional[StripeResponse] = None,
        requestor: "_APIRequestor",
        api_mode: ApiMode,
    ) -> "V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent":
        evt = super()._construct_from(
            values=values,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )
        if hasattr(evt, "data"):
            evt.data = V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEvent.V2CoreHealthIssuingAuthorizationRequestTimeoutResolvedEventData._construct_from(
                values=evt.data,
                last_response=last_response,
                requestor=requestor,
                api_mode=api_mode,
            )
        return evt



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class RateCardSubscriptionUpdateParams(TypedDict):
    metadata: NotRequired[Dict[str, Optional[str]]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from typing import ClassVar, Optional
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._charge import Charge
    from stripe._payment_intent import PaymentIntent
    from stripe.params.radar._early_fraud_warning_list_params import (
        EarlyFraudWarningListParams,
    )
    from stripe.params.radar._early_fraud_warning_retrieve_params import (
        EarlyFraudWarningRetrieveParams,
    )


class EarlyFraudWarning(ListableAPIResource["EarlyFraudWarning"]):
    """
    An early fraud warning indicates that the card issuer has notified us that a
    charge may be fraudulent.

    Related guide: [Early fraud warnings](https://stripe.com/docs/disputes/measuring#early-fraud-warnings)
    """

    OBJECT_NAME: ClassVar[Literal["radar.early_fraud_warning"]] = (
        "radar.early_fraud_warning"
    )
    actionable: bool
    """
    An EFW is actionable if it has not received a dispute and has not been fully refunded. You may wish to proactively refund a charge that receives an EFW, in order to avoid receiving a dispute later.
    """
    charge: ExpandableField["Charge"]
    """
    ID of the charge this early fraud warning is for, optionally expanded.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    fraud_type: str
    """
    The type of fraud labelled by the issuer. One of `card_never_received`, `fraudulent_card_application`, `made_with_counterfeit_card`, `made_with_lost_card`, `made_with_stolen_card`, `misc`, `unauthorized_use_of_card`.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["radar.early_fraud_warning"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    payment_intent: Optional[ExpandableField["PaymentIntent"]]
    """
    ID of the Payment Intent this early fraud warning is for, optionally expanded.
    """

    @classmethod
    def list(
        cls, **params: Unpack["EarlyFraudWarningListParams"]
    ) -> ListObject["EarlyFraudWarning"]:
        """
        Returns a list of early fraud warnings.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["EarlyFraudWarningListParams"]
    ) -> ListObject["EarlyFraudWarning"]:
        """
        Returns a list of early fraud warnings.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["EarlyFraudWarningRetrieveParams"]
    ) -> "EarlyFraudWarning":
        """
        Retrieves the details of an early fraud warning that has previously been created.

        Please refer to the [early fraud warning](https://docs.stripe.com/api#early_fraud_warning_object) object reference for more details.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["EarlyFraudWarningRetrieveParams"]
    ) -> "EarlyFraudWarning":
        """
        Retrieves the details of an early fraud warning that has previously been created.

        Please refer to the [early fraud warning](https://docs.stripe.com/api#early_fraud_warning_object) object reference for more details.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance



from typing import List, Optional


class StripeContext:
    """
    The StripeContext class provides an immutable container and convenience methods for interacting with the `Stripe-Context` header. All methods return a new instance of StripeContext.

    You can use it whenever you're initializing a `StripeClient` or sending `stripe_context` with a request. It's also found in the `EventNotification.context` property.
    """

    def __init__(self, segments: Optional[List[str]] = None):
        self._segments = segments or []

    def push(self, segment: str) -> "StripeContext":
        return StripeContext(self._segments + [segment])

    def pop(self) -> "StripeContext":
        if not self._segments:
            raise ValueError("Cannot pop from an empty StripeContext")

        return StripeContext(self._segments[:-1])

    def __str__(self) -> str:
        return "/".join(self._segments)

    def __repr__(self) -> str:
        return f"StripeContext({self._segments!r})"

    def __eq__(self, other) -> bool:
        if not isinstance(other, StripeContext):
            return False
        return self._segments == other._segments

    @staticmethod
    def parse(context_str: str) -> "StripeContext":
        if not context_str:
            return StripeContext()

        segments = context_str.split("/")
        return StripeContext(segments)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class AccountListParams(TypedDict):
    applied_configurations: NotRequired[List[str]]
    """
    Filter only accounts that have all of the configurations specified. If omitted, returns all accounts regardless of which configurations they have.
    """
    closed: NotRequired[bool]
    """
    Filter by whether the account is closed. If omitted, returns only Accounts that are not closed.
    """
    limit: NotRequired[int]
    """
    The upper limit on the number of accounts returned by the List Account request.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Dict, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Any


class ReportRun(StripeObject):
    """
    The `ReportRun` object represents an instance of a `Report` generated with specific
    run parameters. Once the object is created, Stripe begins processing the report. When
    the report has finished running, it will give you a reference to the results.
    """

    OBJECT_NAME: ClassVar[Literal["v2.reporting.report_run"]] = (
        "v2.reporting.report_run"
    )

    class Result(StripeObject):
        class File(StripeObject):
            class DownloadUrl(StripeObject):
                expires_at: Optional[str]
                """
                The time that the URL expires.
                """
                url: str
                """
                The URL that can be used for accessing the file.
                """

            content_type: Literal["csv", "zip"]
            """
            The content type of the file.
            """
            download_url: DownloadUrl
            """
            A pre-signed URL that allows secure, time-limited access to download the file.
            """
            size: int
            """
            The total size of the file in bytes.
            """
            _inner_class_types = {"download_url": DownloadUrl}

        file: File
        """
        Contains metadata about the file produced by the `ReportRun`, including
        its content type, size, and a URL to download its contents.
        """
        type: Literal["file"]
        """
        The type of the `ReportRun` result.
        """
        _inner_class_types = {"file": File}

    class ResultOptions(StripeObject):
        compress_file: Optional[bool]
        """
        If set, the generated report file will be compressed into a ZIP folder.
        This is useful for reducing file size and download time for large reports.
        """

    class StatusDetails(StripeObject):
        error_code: Optional[
            Literal["file_size_above_limit", "internal_error"]
        ]
        """
        Error code categorizing the reason the `ReportRun` failed.
        """
        error_message: Optional[str]
        """
        Error message with additional details about the failure.
        """

    created: str
    """
    Time at which the object was created.
    """
    id: str
    """
    The unique identifier of the `ReportRun` object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.reporting.report_run"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    report: str
    """
    The unique identifier of the `Report` object which was run.
    """
    report_name: str
    """
    The human-readable name of the `Report` which was run.
    """
    report_parameters: Dict[str, "Any"]
    """
    The parameters used to customize the generation of the report.
    """
    result: Optional[Result]
    """
    Details how to retrieve the results of a successfully completed `ReportRun`.
    """
    result_options: Optional[ResultOptions]
    """
    The options specified for customizing the output file of the `ReportRun`.
    """
    status: Literal["failed", "running", "succeeded"]
    """
    The current status of the `ReportRun`.
    """
    status_details: Dict[str, StatusDetails]
    """
    Additional details about the current state of the `ReportRun`. The field is currently only populated when a `ReportRun`
    is in the `failed` state, providing more information about why the report failed to generate successfully.
    """
    _inner_class_types = {
        "result": Result,
        "result_options": ResultOptions,
        "status_details": StatusDetails,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2._amount import AmountParam
from typing_extensions import NotRequired, TypedDict

_InboundTransferCreateParamsBase = TypedDict(
    "InboundTransferCreateParams",
    {"from": "InboundTransferCreateParamsFrom"},
)


class InboundTransferCreateParams(_InboundTransferCreateParamsBase):
    amount: AmountParam
    """
    The amount, in specified currency, by which the FinancialAccount balance will increase due to the InboundTransfer.
    """
    description: NotRequired[str]
    """
    An optional, freeform description field intended to store metadata.
    """
    to: "InboundTransferCreateParamsTo"
    """
    Object containing details about where the funds will land.
    """


class InboundTransferCreateParamsFrom(TypedDict):
    currency: NotRequired[str]
    """
    An optional currency field used to specify which currency is debited from the Payment Method.
    Since many Payment Methods support only one currency, this field is optional.
    """
    payment_method: str
    """
    ID of the Payment Method using which IBT will be made.
    """


class InboundTransferCreateParamsTo(TypedDict):
    currency: str
    """
    The currency in which funds will land in.
    """
    financial_account: str
    """
    The FinancialAccount that funds will land in.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.capital._financing_offer import FinancingOffer
    from stripe.params.test_helpers.capital._financing_offer_create_params import (
        FinancingOfferCreateParams,
    )
    from stripe.params.test_helpers.capital._financing_offer_refill_params import (
        FinancingOfferRefillParams,
    )


class FinancingOfferService(StripeService):
    def create(
        self,
        params: "FinancingOfferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancingOffer":
        """
        Creates a test financing offer for a connected account.
        """
        return cast(
            "FinancingOffer",
            self._request(
                "post",
                "/v1/test_helpers/capital/financing_offers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "FinancingOfferCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancingOffer":
        """
        Creates a test financing offer for a connected account.
        """
        return cast(
            "FinancingOffer",
            await self._request_async(
                "post",
                "/v1/test_helpers/capital/financing_offers",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def refill(
        self,
        financing_offer: str,
        params: "FinancingOfferRefillParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancingOffer":
        """
        Refills a test financing offer for a connected account.
        """
        return cast(
            "FinancingOffer",
            self._request(
                "post",
                "/v1/test_helpers/capital/financing_offers/{financing_offer}/refill".format(
                    financing_offer=sanitize_id(financing_offer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def refill_async(
        self,
        financing_offer: str,
        params: "FinancingOfferRefillParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FinancingOffer":
        """
        Refills a test financing offer for a connected account.
        """
        return cast(
            "FinancingOffer",
            await self._request_async(
                "post",
                "/v1/test_helpers/capital/financing_offers/{financing_offer}/refill".format(
                    financing_offer=sanitize_id(financing_offer),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2._amount import AmountParam
from typing import Dict
from typing_extensions import Literal, NotRequired, TypedDict

_OutboundPaymentCreateParamsBase = TypedDict(
    "OutboundPaymentCreateParams",
    {"from": "OutboundPaymentCreateParamsFrom"},
)


class OutboundPaymentCreateParams(_OutboundPaymentCreateParamsBase):
    amount: AmountParam
    """
    The "presentment amount" to be sent to the recipient.
    """
    delivery_options: NotRequired["OutboundPaymentCreateParamsDeliveryOptions"]
    """
    Delivery options to be used to send the OutboundPayment.
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the OutboundPayment. Often useful for displaying to users.
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of key-value pairs that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    outbound_payment_quote: NotRequired[str]
    """
    The quote for this OutboundPayment. Only required for countries with regulatory mandates to display fee estimates before OutboundPayment creation.
    """
    recipient_notification: NotRequired[
        "OutboundPaymentCreateParamsRecipientNotification"
    ]
    """
    Details about the notification settings for the OutboundPayment recipient.
    """
    recipient_verification: NotRequired[str]
    """
    The recipient verification id for this OutboundPayment. Only required for countries with regulatory mandates to verify recipient names before OutboundPayment creation.
    """
    to: "OutboundPaymentCreateParamsTo"
    """
    To which payout method to send the OutboundPayment.
    """


class OutboundPaymentCreateParamsDeliveryOptions(TypedDict):
    bank_account: NotRequired[Literal["automatic", "local", "wire"]]
    """
    Open Enum. Method for bank account.
    """


class OutboundPaymentCreateParamsFrom(TypedDict):
    currency: str
    """
    Describes the FinancialAmount's currency drawn from.
    """
    financial_account: str
    """
    The FinancialAccount that funds were pulled from.
    """


class OutboundPaymentCreateParamsRecipientNotification(TypedDict):
    setting: Literal["configured", "none"]
    """
    Closed Enum. Configuration option to enable or disable notifications to recipients.
    Do not send notifications when setting is NONE. Default to account setting when setting is CONFIGURED or not set.
    """


class OutboundPaymentCreateParamsTo(TypedDict):
    currency: NotRequired[str]
    """
    Describes the currency to send to the recipient.
    If included, this currency must match a currency supported by the destination.
    Can be omitted in the following cases:
    - destination only supports one currency
    - destination supports multiple currencies and one of the currencies matches the FA currency
    - destination supports multiple currencies and one of the currencies matches the presentment currency
    Note - when both FA currency and presentment currency are supported, we pick the FA currency to minimize FX.
    """
    payout_method: NotRequired[str]
    """
    The payout method which the OutboundPayment uses to send payout.
    """
    recipient: str
    """
    To which account the OutboundPayment is sent.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class FinancialAddressCreateParams(TypedDict):
    financial_account: str
    """
    The ID of the FinancialAccount the new FinancialAddress should be associated with.
    """
    sepa_bank_account: NotRequired[
        "FinancialAddressCreateParamsSepaBankAccount"
    ]
    """
    Optional SEPA Bank account options, used to configure the type of SEPA Bank account to create, such as the originating country.
    """
    type: Literal["gb_bank_account", "sepa_bank_account", "us_bank_account"]
    """
    The type of FinancialAddress details to provision.
    """


class FinancialAddressCreateParamsSepaBankAccount(TypedDict):
    country: str
    """
    The originating country of the SEPA Bank account.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._singleton_api_resource import SingletonAPIResource
from typing import ClassVar
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.billing.analytics._meter_usage_row import MeterUsageRow
    from stripe.params.billing.analytics._meter_usage_retrieve_params import (
        MeterUsageRetrieveParams,
    )


class MeterUsage(SingletonAPIResource["MeterUsage"]):
    """
    A billing meter usage event represents an aggregated view of a customer's billing meter events within a specified timeframe.
    """

    OBJECT_NAME: ClassVar[Literal["billing.analytics.meter_usage"]] = (
        "billing.analytics.meter_usage"
    )
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["billing.analytics.meter_usage"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    refreshed_at: int
    """
    The timestamp to indicate data freshness, measured in seconds since the Unix epoch.
    """
    rows: ListObject["MeterUsageRow"]

    @classmethod
    def retrieve(
        cls, **params: Unpack["MeterUsageRetrieveParams"]
    ) -> "MeterUsage":
        """
        Returns aggregated meter usage data for a customer within a specified time interval. The data can be grouped by various dimensions and can include multiple meters if specified.
        """
        instance = cls(None, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, **params: Unpack["MeterUsageRetrieveParams"]
    ) -> "MeterUsage":
        """
        Returns aggregated meter usage data for a customer within a specified time interval. The data can be grouped by various dimensions and can include multiple meters if specified.
        """
        instance = cls(None, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def class_url(cls):
        return "/v1/billing/analytics/meter_usage"
